@@ -8,11 +8,13 @@
 //  SPDX-License-Identifier: LGPL-2.0-only
 
 #include "plugin_kategdb.h"
+#include "stackframe_model.h"
 
 #include <QBoxLayout>
 #include <QFile>
 #include <QFontDatabase>
 #include <QKeyEvent>
+#include <QProcess>
 #include <QScrollBar>
 #include <QSplitter>
 #include <QTabWidget>
@@ -21,18 +23,20 @@
 
 #include <KActionCollection>
 #include <KConfigGroup>
-#include <KXMLGUIFactory>
 #include <QAction>
+#include <QClipboard>
+#include <QDir>
 #include <QMenu>
 
 #include <KColorScheme>
 #include <KHistoryComboBox>
 #include <KLocalizedString>
 #include <KPluginFactory>
+#include <KXMLGUIFactory>
 
 #include "debugconfigpage.h"
+#include <KShell>
 #include <KTextEditor/Document>
-#include <QDir>
 #include <ktexteditor/editor.h>
 #include <ktexteditor/view.h>
 
@@ -97,7 +101,6 @@ KatePluginGDBView::KatePluginGDBView(KatePluginGDB *plugin, KTextEditor::MainWin
 {
     m_lastExecUrl = QUrl();
     m_lastExecLine = -1;
-    m_lastExecFrame = 0;
     m_kateApplication = KTextEditor::Editor::instance()->application();
     m_focusOnInput = true;
     m_activeThread = -1;
@@ -115,11 +118,12 @@ KatePluginGDBView::KatePluginGDBView(KatePluginGDB *plugin, KTextEditor::MainWin
                                                           i18n("Locals and Stack")));
 
     m_tabWidget = new QTabWidget(m_toolView.get());
+    m_tabWidget->setDocumentMode(true);
 
     // buttons widget, buttons are initialized at action connections
     m_buttonWidget = new QWidget(m_tabWidget);
     auto buttonsLayout = new QHBoxLayout(m_buttonWidget);
-    buttonsLayout->setContentsMargins(0, 1, 0, 3);
+    buttonsLayout->setContentsMargins(0, 0, 0, 0);
     m_tabWidget->setCornerWidget(m_buttonWidget);
 
     // Output
@@ -139,14 +143,14 @@ KatePluginGDBView::KatePluginGDBView(KatePluginGDB *plugin, KTextEditor::MainWin
 
     // input
     m_inputArea = new KHistoryComboBox(true);
-    connect(m_inputArea, static_cast<void (KHistoryComboBox::*)(const QString &)>(&KHistoryComboBox::returnPressed), this, &KatePluginGDBView::slotSendCommand);
-    QHBoxLayout *inputLayout = new QHBoxLayout();
+    connect(m_inputArea, &KHistoryComboBox::returnPressed, this, &KatePluginGDBView::slotSendCommand);
+    auto *inputLayout = new QHBoxLayout();
     inputLayout->addWidget(m_inputArea, 10);
     inputLayout->setContentsMargins(0, 0, 0, 0);
     m_outputArea->setFocusProxy(m_inputArea); // take the focus from the outputArea
 
     m_gdbPage = new QWidget();
-    QVBoxLayout *layout = new QVBoxLayout(m_gdbPage);
+    auto *layout = new QVBoxLayout(m_gdbPage);
     layout->addWidget(m_outputArea);
     layout->addLayout(inputLayout);
     layout->setStretch(0, 10);
@@ -154,28 +158,32 @@ KatePluginGDBView::KatePluginGDBView(KatePluginGDB *plugin, KTextEditor::MainWin
     layout->setSpacing(0);
 
     // stack page
-    QWidget *stackContainer = new QWidget();
-    QVBoxLayout *stackLayout = new QVBoxLayout(stackContainer);
+    auto *stackContainer = new QWidget();
+    auto *stackLayout = new QVBoxLayout(stackContainer);
     m_threadCombo = new QComboBox();
-    m_stackTree = new QTreeWidget();
+    m_stackTree = new QTreeView();
     stackLayout->addWidget(m_threadCombo);
     stackLayout->addWidget(m_stackTree);
     stackLayout->setStretch(0, 10);
     stackLayout->setContentsMargins(0, 0, 0, 0);
     stackLayout->setSpacing(0);
-    QStringList headers;
-    headers << QStringLiteral("  ") << i18nc("Column label (frame number)", "Nr") << i18nc("Column label", "Frame");
-    m_stackTree->setHeaderLabels(headers);
+
+    m_stackTree->setModel(new StackFrameModel(this));
+
     m_stackTree->setRootIsDecorated(false);
     m_stackTree->resizeColumnToContents(0);
     m_stackTree->resizeColumnToContents(1);
     m_stackTree->setAutoScroll(false);
-    connect(m_stackTree, &QTreeWidget::itemActivated, this, &KatePluginGDBView::stackFrameSelected);
+    m_stackTree->setUniformRowHeights(true);
 
-    connect(m_threadCombo, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged), this, &KatePluginGDBView::threadSelected);
+    connect(m_stackTree, &QTreeView::activated, this, &KatePluginGDBView::stackFrameSelected);
+    m_stackTree->setContextMenuPolicy(Qt::CustomContextMenu);
+    connect(m_stackTree, &QTreeWidget::customContextMenuRequested, this, &KatePluginGDBView::onStackTreeContextMenuRequest);
 
-    QWidget *variableContainer = new QWidget();
-    QVBoxLayout *variableLayout = new QVBoxLayout(variableContainer);
+    connect(m_threadCombo, &QComboBox::currentIndexChanged, this, &KatePluginGDBView::threadSelected);
+
+    auto *variableContainer = new QWidget();
+    auto *variableLayout = new QVBoxLayout(variableContainer);
     m_scopeCombo = new QComboBox();
     connect(m_scopeCombo, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &KatePluginGDBView::scopeSelected);
     m_localsView = new LocalsView();
@@ -185,10 +193,18 @@ KatePluginGDBView::KatePluginGDBView(KatePluginGDB *plugin, KTextEditor::MainWin
     variableLayout->setContentsMargins(0, 0, 0, 0);
     variableLayout->setSpacing(0);
 
-    QSplitter *locStackSplitter = new QSplitter(m_localsStackToolView.get());
+    auto *locStackSplitter = m_localsStackSplitter = new QSplitter(m_localsStackToolView.get());
     locStackSplitter->addWidget(variableContainer);
     locStackSplitter->addWidget(stackContainer);
-    locStackSplitter->setOrientation(Qt::Vertical);
+
+    const auto pos = toolviewPosition(m_localsStackToolView.get());
+    const Qt::Orientation orientation = (pos == KTextEditor::MainWindow::Bottom || pos == KTextEditor::MainWindow::Top) ? Qt::Horizontal : Qt::Vertical;
+    locStackSplitter->setOrientation(orientation);
+
+    connect(m_mainWin->window(),
+            SIGNAL(toolViewMoved(QWidget *, KTextEditor::MainWindow::ToolViewPosition)),
+            this,
+            SLOT(onToolViewMoved(QWidget *, KTextEditor::MainWindow::ToolViewPosition)));
 
     m_ioView = std::make_unique<IOView>();
 
@@ -241,91 +257,103 @@ KatePluginGDBView::KatePluginGDBView(KatePluginGDB *plugin, KTextEditor::MainWin
     });
 
     connect(m_localsView, &LocalsView::localsVisible, m_backend, &BackendInterface::slotQueryLocals);
+    connect(m_localsView, &LocalsView::requestVariable, m_backend, &BackendInterface::requestVariable);
 
+    connect(m_backend, &BackendInterface::debuggeeRequiresTerminal, this, &KatePluginGDBView::requestRunInTerminal);
+
+    auto ac = actionCollection();
     // Actions
-    m_targetSelectAction = actionCollection()->add<KSelectAction>(QStringLiteral("targets"));
+    m_targetSelectAction = ac->add<KSelectAction>(QStringLiteral("targets"));
     m_targetSelectAction->setText(i18n("Targets"));
     m_targetSelectAction->setEnabled(true);
     connect(m_targetSelectAction->menu(), &QMenu::aboutToShow, this, &KatePluginGDBView::initDebugToolview);
 
-    QAction *a = actionCollection()->addAction(QStringLiteral("debug"));
+    QAction *a = ac->addAction(QStringLiteral("debug"));
     a->setText(i18n("Start Debugging"));
     a->setIcon(QIcon::fromTheme(QStringLiteral("debug-run")));
-    actionCollection()->setDefaultShortcut(a, QKeySequence((Qt::CTRL | Qt::SHIFT | Qt::Key_F7)));
+    KActionCollection::setDefaultShortcut(a, QKeySequence((Qt::CTRL | Qt::SHIFT | Qt::Key_F7)));
     connect(a, &QAction::triggered, this, &KatePluginGDBView::slotDebug);
     // This button is handled differently due to it having multiple actions
     m_continueButton = new QToolButton();
     m_continueButton->setAutoRaise(true);
     buttonsLayout->addWidget(m_continueButton);
 
-    a = actionCollection()->addAction(QStringLiteral("kill"));
+    a = ac->addAction(QStringLiteral("kill"));
     a->setText(i18n("Kill / Stop Debugging"));
     a->setIcon(QIcon::fromTheme(QStringLiteral("media-playback-stop")));
-    actionCollection()->setDefaultShortcut(a, QKeySequence((Qt::ALT | Qt::Key_F7)));
+    KActionCollection::setDefaultShortcut(a, QKeySequence((Qt::ALT | Qt::Key_F7)));
     connect(a, &QAction::triggered, m_backend, &BackendInterface::slotKill);
     buttonsLayout->addWidget(createDebugButton(a));
 
-    a = actionCollection()->addAction(QStringLiteral("continue"));
+    a = ac->addAction(QStringLiteral("continue"));
     a->setText(i18n("Continue"));
     a->setIcon(QIcon::fromTheme(QStringLiteral("media-playback-start")));
-    actionCollection()->setDefaultShortcut(a, QKeySequence((Qt::CTRL | Qt::Key_F7)));
+    KActionCollection::setDefaultShortcut(a, QKeySequence((Qt::CTRL | Qt::Key_F7)));
     connect(a, &QAction::triggered, m_backend, &BackendInterface::slotContinue);
 
-    a = actionCollection()->addAction(u"toggle_breakpoint"_s);
+    a = ac->addAction(u"toggle_breakpoint"_s);
     a->setText(i18n("Toggle Breakpoint / Break"));
     a->setIcon(QIcon::fromTheme(QStringLiteral("media-playback-pause")));
-    actionCollection()->setDefaultShortcut(a, QKeySequence((Qt::SHIFT | Qt::Key_F11)));
+    KActionCollection::setDefaultShortcut(a, QKeySequence((Qt::SHIFT | Qt::Key_F11)));
     connect(a, &QAction::triggered, this, &KatePluginGDBView::slotToggleBreakpoint);
     buttonsLayout->addWidget(createDebugButton(a));
 
-    a = actionCollection()->addAction(u"clear_all_breakpoints"_s);
+    a = ac->addAction(u"clear_all_breakpoints"_s);
     a->setText(i18n("Clear All Breakpoints"));
     a->setIcon(QIcon::fromTheme(QStringLiteral("edit-clear-all")));
     connect(a, &QAction::triggered, this, &KatePluginGDBView::clearMarks);
 
-    a = actionCollection()->addAction(QStringLiteral("step_in"));
+    a = ac->addAction(QStringLiteral("step_in"));
     a->setText(i18n("Step In"));
     a->setIcon(QIcon::fromTheme(QStringLiteral("debug-step-into")));
-    actionCollection()->setDefaultShortcut(a, QKeySequence((Qt::Key_F10)));
+    KActionCollection::setDefaultShortcut(a, QKeySequence((Qt::Key_F10)));
     connect(a, &QAction::triggered, m_backend, &BackendInterface::slotStepInto);
     buttonsLayout->addWidget(createDebugButton(a));
 
-    a = actionCollection()->addAction(QStringLiteral("step_over"));
+    a = ac->addAction(QStringLiteral("step_over"));
     a->setText(i18n("Step Over"));
     a->setIcon(QIcon::fromTheme(QStringLiteral("debug-step-over")));
-    actionCollection()->setDefaultShortcut(a, QKeySequence((Qt::Key_F9)));
+    KActionCollection::setDefaultShortcut(a, QKeySequence((Qt::Key_F9)));
     connect(a, &QAction::triggered, m_backend, &BackendInterface::slotStepOver);
     buttonsLayout->addWidget(createDebugButton(a));
 
-    a = actionCollection()->addAction(QStringLiteral("step_out"));
+    a = ac->addAction(QStringLiteral("step_out"));
     a->setText(i18n("Step Out"));
     a->setIcon(QIcon::fromTheme(QStringLiteral("debug-step-out")));
-    actionCollection()->setDefaultShortcut(a, QKeySequence((Qt::SHIFT | Qt::Key_F10)));
+    KActionCollection::setDefaultShortcut(a, QKeySequence((Qt::SHIFT | Qt::Key_F10)));
     connect(a, &QAction::triggered, m_backend, &BackendInterface::slotStepOut);
     buttonsLayout->addWidget(createDebugButton(a));
 
-    a = actionCollection()->addAction(QStringLiteral("run_to_cursor"));
+    a = ac->addAction(QStringLiteral("run_to_cursor"));
     a->setText(i18n("Run To Cursor"));
     a->setIcon(QIcon::fromTheme(QStringLiteral("debug-run-cursor")));
-    actionCollection()->setDefaultShortcut(a, QKeySequence((Qt::SHIFT | Qt::Key_F9)));
+    KActionCollection::setDefaultShortcut(a, QKeySequence((Qt::SHIFT | Qt::Key_F9)));
     connect(a, &QAction::triggered, this, &KatePluginGDBView::slotRunToCursor);
     buttonsLayout->addWidget(createDebugButton(a));
 
-    a = actionCollection()->addAction(QStringLiteral("rerun"));
+    a = ac->addAction(QStringLiteral("rerun"));
     a->setText(i18n("Restart Debugging"));
     a->setIcon(QIcon::fromTheme(QStringLiteral("view-refresh")));
-    actionCollection()->setDefaultShortcut(a, QKeySequence((Qt::CTRL | Qt::SHIFT | Qt::Key_F7)));
+    KActionCollection::setDefaultShortcut(a, QKeySequence((Qt::CTRL | Qt::SHIFT | Qt::Key_F7)));
     connect(a, &QAction::triggered, this, &KatePluginGDBView::slotRestart);
     buttonsLayout->addWidget(createDebugButton(a));
 
-    a = actionCollection()->addAction(QStringLiteral("debug_hot_reload"));
+    a = ac->addAction(QStringLiteral("clear_debugoutput"));
+    a->setText(i18n("Clear Output"));
+    a->setIcon(QIcon::fromTheme(QStringLiteral("edit-clear")));
+    connect(a, &QAction::triggered, this, [this] {
+        m_outputArea->clear();
+    });
+    buttonsLayout->addWidget(createDebugButton(a));
+
+    a = ac->addAction(QStringLiteral("debug_hot_reload"));
     a->setText(i18n("Hot Reload"));
     a->setIcon(QIcon::fromTheme(QStringLiteral("exception")));
     a->setVisible(false);
     connect(a, &QAction::triggered, m_backend, &Backend::slotHotReload);
     buttonsLayout->addWidget(createDebugButton(a));
 
-    a = actionCollection()->addAction(QStringLiteral("debug_hot_restart"));
+    a = ac->addAction(QStringLiteral("debug_hot_restart"));
     a->setText(i18n("Hot Restart"));
     a->setIcon(QIcon::fromTheme(QStringLiteral("edit-reset")));
     a->setVisible(false);
@@ -334,7 +362,7 @@ KatePluginGDBView::KatePluginGDBView(KatePluginGDB *plugin, KTextEditor::MainWin
 
     buttonsLayout->addStretch();
 
-    m_hotReloadOnSaveAction = a = actionCollection()->addAction(QStringLiteral("debug_hot_reload_on_save"));
+    m_hotReloadOnSaveAction = a = ac->addAction(QStringLiteral("debug_hot_reload_on_save"));
     a->setText(i18n("Enable Hot Reload on Save"));
     a->setVisible(false);
     a->setCheckable(true);
@@ -343,19 +371,19 @@ KatePluginGDBView::KatePluginGDBView(KatePluginGDB *plugin, KTextEditor::MainWin
         enableHotReloadOnSave(e ? m_mainWin->activeView() : nullptr);
     });
 
-    a = actionCollection()->addAction(QStringLiteral("move_pc"));
+    a = ac->addAction(QStringLiteral("move_pc"));
     a->setText(i18nc("Move Program Counter (next execution)", "Move PC"));
     connect(a, &QAction::triggered, this, &KatePluginGDBView::slotMovePC);
-    actionCollection()->setDefaultShortcut(a, QKeySequence((Qt::ALT | Qt::Key_F9)));
+    KActionCollection::setDefaultShortcut(a, QKeySequence((Qt::ALT | Qt::Key_F9)));
 
-    a = actionCollection()->addAction(QStringLiteral("print_value"));
+    a = ac->addAction(QStringLiteral("print_value"));
     a->setText(i18n("Print Value"));
     a->setIcon(QIcon::fromTheme(QStringLiteral("document-preview")));
     connect(a, &QAction::triggered, this, &KatePluginGDBView::slotValue);
 
     // popup context m_menu
     m_menu = new KActionMenu(i18n("Debug"), this);
-    actionCollection()->addAction(QStringLiteral("popup_gdb"), m_menu);
+    ac->addAction(QStringLiteral("popup_gdb"), m_menu);
     connect(m_menu->menu(), &QMenu::aboutToShow, this, &KatePluginGDBView::aboutToShowMenu);
 
     m_breakpoint = m_menu->menu()->addAction(u"popup_breakpoint"_s, this, &KatePluginGDBView::slotToggleBreakpoint);
@@ -409,21 +437,6 @@ void KatePluginGDBView::writeSessionConfig(KConfigGroup &config)
 void KatePluginGDBView::slotDebug()
 {
     initDebugToolview();
-#ifndef Q_OS_WIN
-    disconnect(m_ioView.get(), &IOView::stdOutText, nullptr, nullptr);
-    disconnect(m_ioView.get(), &IOView::stdErrText, nullptr, nullptr);
-    if (m_configView->showIOTab()) {
-        connect(m_ioView.get(), &IOView::stdOutText, m_ioView.get(), &IOView::addStdOutText);
-        connect(m_ioView.get(), &IOView::stdErrText, m_ioView.get(), &IOView::addStdErrText);
-    } else {
-        connect(m_ioView.get(), &IOView::stdOutText, this, &KatePluginGDBView::addOutputText);
-        connect(m_ioView.get(), &IOView::stdErrText, this, &KatePluginGDBView::addErrorText);
-    }
-    QStringList ioFifos;
-    ioFifos << m_ioView->stdinFifo();
-    ioFifos << m_ioView->stdoutFifo();
-    ioFifos << m_ioView->stderrFifo();
-#endif
 
     m_outputArea->clear();
 
@@ -435,15 +448,7 @@ void KatePluginGDBView::slotDebug()
     m_scopeCombo->clear();
     m_localsView->clear();
 
-#ifndef Q_OS_WIN
-    if (m_configView->debuggerIsGDB()) {
-        m_backend->runDebugger(m_configView->currentGDBTarget(), ioFifos);
-    } else {
-        m_backend->runDebugger(m_configView->currentDAPTarget(true));
-    }
-#else
     m_backend->runDebugger(m_configView->currentDAPTarget(true));
-#endif
 }
 
 void KatePluginGDBView::slotRestart()
@@ -495,15 +500,12 @@ void KatePluginGDBView::slotToggleBreakpoint()
         bool added = true;
         m_backend->toggleBreakpoint(currURL, line, &added);
 
-        // We don't support setting breakpoints for gdb when the debugger is not running'
-        if (!m_backend->debuggerRunning() && m_configView && m_configView->debuggerIsGDB()) {
-            return;
-        }
-
-        if (added) {
-            slotBreakpointSet(currURL, line);
-        } else {
-            slotBreakpointCleared(currURL, line);
+        if (!m_backend->debuggerRunning()) {
+            if (added) {
+                slotBreakpointSet(currURL, line);
+            } else {
+                slotBreakpointCleared(currURL, line);
+            }
         }
     }
 }
@@ -578,16 +580,18 @@ void KatePluginGDBView::slotGoTo(const QUrl &url, int lineNum)
 void KatePluginGDBView::enableDebugActions(bool enable)
 {
     const bool canMove = m_backend->canMove();
-    actionCollection()->action(QStringLiteral("step_in"))->setEnabled(enable && canMove);
-    actionCollection()->action(QStringLiteral("step_over"))->setEnabled(enable && canMove);
-    actionCollection()->action(QStringLiteral("step_out"))->setEnabled(enable && canMove);
-    actionCollection()->action(QStringLiteral("move_pc"))->setEnabled(enable && m_backend->supportsMovePC());
-    actionCollection()->action(QStringLiteral("run_to_cursor"))->setEnabled(enable && m_backend->supportsRunToCursor());
-    actionCollection()->action(QStringLiteral("popup_gdb"))->setEnabled(enable);
-    actionCollection()->action(QStringLiteral("continue"))->setEnabled(enable && m_backend->canContinue());
-    actionCollection()->action(QStringLiteral("print_value"))->setEnabled(enable);
+    auto ac = actionCollection();
+    ac->action(QStringLiteral("step_in"))->setEnabled(enable && canMove);
+    ac->action(QStringLiteral("step_over"))->setEnabled(enable && canMove);
+    ac->action(QStringLiteral("step_out"))->setEnabled(enable && canMove);
+    ac->action(QStringLiteral("move_pc"))->setEnabled(enable && m_backend->supportsMovePC());
+    ac->action(QStringLiteral("run_to_cursor"))->setEnabled(enable && m_backend->supportsRunToCursor());
+    ac->action(QStringLiteral("popup_gdb"))->setEnabled(enable);
+    ac->action(QStringLiteral("continue"))->setEnabled(enable && m_backend->canContinue());
+    ac->action(QStringLiteral("print_value"))->setEnabled(enable);
+    ac->action(QStringLiteral("clear_debugoutput"))->setEnabled(enable);
 
-    auto a = actionCollection()->action(QStringLiteral("debug_hot_reload"));
+    auto a = ac->action(QStringLiteral("debug_hot_reload"));
     a->setVisible(enable && m_backend->canHotReload());
     a->setEnabled(enable && m_backend->canHotReload());
 
@@ -603,20 +607,20 @@ void KatePluginGDBView::enableDebugActions(bool enable)
         disconnect(m_mainWin, &KTextEditor::MainWindow::viewChanged, this, &KatePluginGDBView::enableHotReloadOnSave);
     }
 
-    a = actionCollection()->action(QStringLiteral("debug_hot_restart"));
+    a = ac->action(QStringLiteral("debug_hot_restart"));
     a->setVisible(enable && m_backend->canHotRestart());
     a->setEnabled(enable && m_backend->canHotRestart());
 
     m_breakpoint->setEnabled(m_backend->debuggerRunning());
 
     const QIcon pauseToggle = m_backend->canContinue() ? QIcon::fromTheme(u"media-record"_s) : QIcon::fromTheme(u"media-playback-pause"_s);
-    actionCollection()->action(u"toggle_breakpoint"_s)->setIcon(pauseToggle);
-    actionCollection()->action(QStringLiteral("kill"))->setEnabled(m_backend->debuggerRunning());
-    actionCollection()->action(QStringLiteral("rerun"))->setEnabled(m_backend->debuggerRunning());
+    ac->action(u"toggle_breakpoint"_s)->setIcon(pauseToggle);
+    ac->action(QStringLiteral("kill"))->setEnabled(m_backend->debuggerRunning());
+    ac->action(QStringLiteral("rerun"))->setEnabled(m_backend->debuggerRunning());
 
     // Combine start and continue button to same button
-    m_continueButton->removeAction(actionCollection()->action(!enable ? QStringLiteral("continue") : QStringLiteral("debug")));
-    m_continueButton->setDefaultAction(actionCollection()->action(enable ? QStringLiteral("continue") : QStringLiteral("debug")));
+    m_continueButton->removeAction(ac->action(!enable ? QStringLiteral("continue") : QStringLiteral("debug")));
+    m_continueButton->setDefaultAction(ac->action(enable ? QStringLiteral("continue") : QStringLiteral("debug")));
     m_continueButton->setIcon(QIcon::fromTheme(QStringLiteral("media-playback-start")));
 
     m_inputArea->setEnabled(enable && !m_backend->debuggerBusy());
@@ -666,7 +670,8 @@ void KatePluginGDBView::programEnded()
 
     // don't set the execution mark on exit
     m_lastExecLine = -1;
-    m_stackTree->clear();
+    static_cast<StackFrameModel *>(m_stackTree->model())->setFrames({});
+    static_cast<StackFrameModel *>(m_stackTree->model())->setActiveFrame(-1);
     m_scopeCombo->clear();
     m_localsView->clear();
     m_threadCombo->clear();
@@ -713,26 +718,11 @@ void KatePluginGDBView::slotSendCommand()
     sb->setValue(sb->maximum());
 }
 
-void KatePluginGDBView::insertStackFrame(int level, const QString &info)
+void KatePluginGDBView::insertStackFrame(const QList<dap::StackFrame> &frames)
 {
-    if (level < 0) {
-        m_stackTree->resizeColumnToContents(2);
-        return;
-    }
-
-    if (level == 0) {
-        m_stackTree->clear();
-    }
-    QStringList columns;
-    columns << QStringLiteral("  "); // icon place holder
-    columns << QString::number(level);
-    int lastSpace = info.lastIndexOf(QLatin1Char(' '));
-    QString shortInfo = info.mid(lastSpace);
-    columns << shortInfo;
-
-    QTreeWidgetItem *item = new QTreeWidgetItem(columns);
-    item->setToolTip(2, QStringLiteral("<qt>%1<qt>").arg(info));
-    m_stackTree->insertTopLevelItem(level, item);
+    auto model = static_cast<StackFrameModel *>(m_stackTree->model());
+    model->setFrames(frames);
+    m_stackTree->resizeColumnToContents(StackFrameModel::Column::Path);
 }
 
 void KatePluginGDBView::stackFrameSelected()
@@ -742,34 +732,26 @@ void KatePluginGDBView::stackFrameSelected()
 
 void KatePluginGDBView::stackFrameChanged(int level)
 {
-    QTreeWidgetItem *current = m_stackTree->topLevelItem(m_lastExecFrame);
-    QTreeWidgetItem *next = m_stackTree->topLevelItem(level);
-
-    if (current) {
-        current->setIcon(0, QIcon());
-    }
-    if (next) {
-        next->setIcon(0, QIcon::fromTheme(QStringLiteral("arrow-right")));
-    }
-    m_lastExecFrame = level;
+    auto model = static_cast<StackFrameModel *>(m_stackTree->model());
+    model->setActiveFrame(level);
 }
 
 void KatePluginGDBView::insertScopes(const QList<dap::Scope> &scopes, std::optional<int> activeId)
 {
+    const int currentIndex = m_scopeCombo->currentIndex();
+    Q_UNUSED(activeId)
+
     m_scopeCombo->clear();
 
-    int selected = -1;
-    int index = 0;
     for (const auto &scope : scopes) {
         QString name = scope.expensive.value_or(false) ? QStringLiteral("%1!").arg(scope.name) : scope.name;
-        if (activeId && (activeId == scope.variablesReference)) {
-            selected = index;
-        }
         m_scopeCombo->addItem(QIcon::fromTheme(QStringLiteral("")).pixmap(10, 10), scope.name, scope.variablesReference);
-        ++index;
     }
-    if (selected >= 0) {
-        m_scopeCombo->setCurrentIndex(selected);
+
+    if (currentIndex >= 0 && currentIndex < scopes.size()) {
+        m_scopeCombo->setCurrentIndex(currentIndex);
+    } else if (m_scopeCombo->count() > 0) {
+        m_scopeCombo->setCurrentIndex(0);
     }
 }
 
@@ -782,6 +764,8 @@ void KatePluginGDBView::scopeSelected(int scope)
 
 void KatePluginGDBView::onThreads(const QList<dap::Thread> &threads)
 {
+    // We dont want to signal thread change while we are modifying the combo
+    disconnect(m_threadCombo, &QComboBox::currentIndexChanged, this, &KatePluginGDBView::threadSelected);
     m_threadCombo->clear();
     int activeThread = m_activeThread;
     m_activeThread = -1;
@@ -801,6 +785,7 @@ void KatePluginGDBView::onThreads(const QList<dap::Thread> &threads)
         m_threadCombo->addItem(icon, name, thread.id);
     }
 
+    connect(m_threadCombo, &QComboBox::currentIndexChanged, this, &KatePluginGDBView::threadSelected);
     // try to set an active thread
     if (m_threadCombo->count() > 0) {
         int idx = 0; // use first thread
@@ -920,16 +905,6 @@ void KatePluginGDBView::initDebugToolview()
     m_tabWidget->addTab(m_configView, i18nc("Tab label", "Settings"));
     m_tabWidget->setCurrentWidget(m_configView); // initially show config
 
-    connect(m_configView, &ConfigView::configChanged, this, [this]() {
-        if (!m_configView->debuggerIsGDB())
-            return;
-
-        GDBTargetConf config = m_configView->currentGDBTarget();
-        if (m_backend->targetName() == config.targetName) {
-            m_backend->setFileSearchPaths(config.srcPaths);
-        }
-    });
-
     m_configView->readConfig(m_sessionConfig);
 }
 
@@ -1018,7 +993,7 @@ void KatePluginGDBView::addErrorText(QString const &text)
 bool KatePluginGDBView::eventFilter(QObject *obj, QEvent *event)
 {
     if (event->type() == QEvent::KeyPress) {
-        QKeyEvent *ke = static_cast<QKeyEvent *>(event);
+        auto *ke = static_cast<QKeyEvent *>(event);
         if ((obj == m_toolView.get()) && (ke->key() == Qt::Key_Escape)) {
             m_mainWin->hideToolView(m_toolView.get());
             event->accept();
@@ -1032,19 +1007,22 @@ bool KatePluginGDBView::eventFilter(QObject *obj, QEvent *event)
 
 void KatePluginGDBView::handleEsc(QEvent *e)
 {
-    if (!m_mainWin || !m_toolView) {
+    if (!m_mainWin) {
         return;
     }
 
-    QKeyEvent *k = static_cast<QKeyEvent *>(e);
+    auto *k = static_cast<QKeyEvent *>(e);
     if (k->key() == Qt::Key_Escape && k->modifiers() == Qt::NoModifier) {
-        if (m_toolView->isVisible()) {
+        if (m_toolView && m_toolView->isVisible()) {
             m_mainWin->hideToolView(m_toolView.get());
+        } else if (m_localsStackToolView && m_localsStackToolView->isVisible()
+                   && toolviewPosition(m_localsStackToolView.get()) == KTextEditor::MainWindow::Bottom) {
+            m_mainWin->hideToolView(m_localsStackToolView.get());
         }
     }
 }
 
-void KatePluginGDBView::enableBreakpointMarks(KTextEditor::Document *document)
+void KatePluginGDBView::enableBreakpointMarks(KTextEditor::Document *document) const
 {
     if (document) {
         document->setEditableMarks(document->editableMarks() | KTextEditor::Document::BreakpointActive);
@@ -1109,7 +1087,7 @@ void KatePluginGDBView::displayMessage(const QString &msg, KTextEditor::Message:
 
 QToolButton *KatePluginGDBView::createDebugButton(QAction *action)
 {
-    QToolButton *button = new QToolButton(m_buttonWidget);
+    auto *button = new QToolButton(m_buttonWidget);
     button->setDefaultAction(action);
     button->setAutoRaise(true);
     connect(action, &QAction::visibleChanged, button, [button, action] {
@@ -1119,5 +1097,102 @@ QToolButton *KatePluginGDBView::createDebugButton(QAction *action)
     return button;
 }
 
+void KatePluginGDBView::onStackTreeContextMenuRequest(QPoint pos)
+{
+    QMenu menu(m_stackTree);
+    auto model = m_stackTree->model();
+
+    auto a = menu.addAction(i18n("Copy Stack Trace"));
+    connect(a, &QAction::triggered, m_stackTree, [model] {
+        QString text;
+        for (int i = 0; i < model->rowCount(); ++i) {
+            QString no = model->index(i, StackFrameModel::Column::Number).data().toString();
+            QString func = model->index(i, StackFrameModel::Column::Func).data().toString();
+            QString path = model->index(i, StackFrameModel::Column::Path).data().toString();
+            text.append(QStringLiteral("#%1  %2  %3\n").arg(no, func, path));
+        }
+        qApp->clipboard()->setText(text);
+    });
+
+    auto index = m_stackTree->currentIndex();
+    if (index.isValid()) {
+        auto frame = index.data(StackFrameModel::StackFrameRole).value<dap::StackFrame>();
+        if (frame.source) {
+            auto url = QUrl::fromLocalFile(frame.source->path);
+            int line = frame.line - 1;
+            if (url.isValid()) {
+                auto a = menu.addAction(i18n("Open Location"));
+                connect(a, &QAction::triggered, m_stackTree, [this, url, line] {
+                    auto view = m_mainWin->openUrl(url);
+                    if (line >= 0) {
+                        view->setCursorPosition({line, 0});
+                    }
+                });
+            }
+        }
+
+        auto a = menu.addAction(i18n("Copy Location"));
+        QString location = QStringLiteral("%1:%2").arg(frame.source->path).arg(frame.line);
+        connect(a, &QAction::triggered, m_stackTree, [location] {
+            qApp->clipboard()->setText(location);
+        });
+    }
+
+    menu.exec(m_stackTree->viewport()->mapToGlobal(pos));
+}
+
+void KatePluginGDBView::requestRunInTerminal(const dap::RunInTerminalRequestArguments &args, const dap::Client::ProcessInTerminal &notifyCreation)
+{
+    if (!args.args.isEmpty()) {
+        auto *terminalJob = new KTerminalLauncherJob(KShell::joinArgs(args.args));
+        terminalJob->setWorkingDirectory(args.cwd);
+
+        /*
+         * Environment key-value pairs that are added to or removed from the default environment.
+         *
+         * env?: { [key: string]: string | null; };
+         */
+        QProcessEnvironment env(QProcessEnvironment::InheritFromParent);
+        if (args.env) {
+            for (auto item = args.env->cbegin(); item != args.env->cend(); ++item) {
+                const auto &value = item.value();
+                if (value) {
+                    env.insert(item.key(), *value);
+                } else {
+                    env.remove(item.key());
+                }
+            }
+        }
+        terminalJob->setProcessEnvironment(env);
+        connect(terminalJob, &KJob::result, [notifyCreation](KJob *job) {
+            notifyCreation(job->error() == 0, std::nullopt, std::nullopt);
+        });
+        terminalJob->start();
+    } else {
+        // notify error
+        notifyCreation(false, std::nullopt, std::nullopt);
+    }
+}
+
+void KatePluginGDBView::onToolViewMoved(QWidget *toolview, KTextEditor::MainWindow::ToolViewPosition newPos)
+{
+    if (toolview != m_localsStackToolView.get()) {
+        return;
+    }
+    auto orientation = m_localsStackSplitter->orientation();
+    const Qt::Orientation newOrientation =
+        (newPos == KTextEditor::MainWindow::Bottom || newPos == KTextEditor::MainWindow::Top) ? Qt::Horizontal : Qt::Vertical;
+    if (orientation != newOrientation) {
+        m_localsStackSplitter->setOrientation(newOrientation);
+    }
+}
+
+KTextEditor::MainWindow::ToolViewPosition KatePluginGDBView::toolviewPosition(QWidget *toolview) const
+{
+    KTextEditor::MainWindow::ToolViewPosition pos;
+    QMetaObject::invokeMethod(m_mainWin->window(), "toolViewPosition", qReturnArg(pos), toolview);
+    return pos;
+}
+
 #include "moc_plugin_kategdb.cpp"
 #include "plugin_kategdb.moc"
