@@ -72,9 +72,8 @@ using namespace Qt::Literals::StringLiterals;
 
 K_PLUGIN_FACTORY_WITH_JSON(KateBuildPluginFactory, "katebuildplugin.json", registerPlugin<KateBuildPlugin>();)
 
-static const QString DiagnosticsPrefix = QStringLiteral("katebuild");
-static const QString ConfigAllowedCommands = QStringLiteral("AllowedCommandLines");
-static const QString ConfigBlockedCommands = QStringLiteral("BlockedCommandLines");
+static const char ConfigAllowedCommands[] = "AllowedCommandLines";
+static const char ConfigBlockedCommands[] = "BlockedCommandLines";
 
 namespace
 {
@@ -175,13 +174,6 @@ static void terminateProcess(KProcess &proc)
 }
 #endif
 
-struct ItemData {
-    // ensure destruction, but not inadvertently so by a variant value copy
-    std::shared_ptr<KTextEditor::MovingCursor> cursor;
-};
-
-Q_DECLARE_METATYPE(ItemData)
-
 /******************************************************************/
 KateBuildPlugin::KateBuildPlugin(QObject *parent, const VariantList &)
     : KTextEditor::Plugin(parent)
@@ -208,7 +200,7 @@ KTextEditor::ConfigPage *KateBuildPlugin::configPage(int number, QWidget *parent
         return nullptr;
     }
 
-    KateBuildConfigPage *configPage = new KateBuildConfigPage(this, parent);
+    auto *configPage = new KateBuildConfigPage(this, parent);
     return configPage;
 }
 
@@ -218,6 +210,7 @@ void KateBuildPlugin::readConfig()
     KConfigGroup config(KSharedConfig::openConfig(), QStringLiteral("BuildConfig"));
     m_addDiagnostics = config.readEntry(QStringLiteral("UseDiagnosticsOutput"), true);
     m_autoSwitchToOutput = config.readEntry(QStringLiteral("AutoSwitchToOutput"), true);
+    m_showBuildProgress = config.readEntry("ShowBuildProgress", false);
 
     // read allow + block lists as two separate keys, let block always win
     const auto allowed = config.readEntry(ConfigAllowedCommands, QStringList());
@@ -237,9 +230,11 @@ void KateBuildPlugin::writeConfig() const
     KConfigGroup config(KSharedConfig::openConfig(), QStringLiteral("BuildConfig"));
     config.writeEntry("UseDiagnosticsOutput", m_addDiagnostics);
     config.writeEntry("AutoSwitchToOutput", m_autoSwitchToOutput);
+    config.writeEntry("ShowBuildProgress", m_showBuildProgress);
 
     // write allow + block lists as two separate keys
-    QStringList allowed, blocked;
+    QStringList allowed;
+    QStringList blocked;
     for (const auto &it : m_commandLineToAllowedState) {
         if (it.second) {
             allowed.push_back(it.first);
@@ -256,17 +251,13 @@ KateBuildView::KateBuildView(KateBuildPlugin *plugin, KTextEditor::MainWindow *m
     : QObject(mw)
     , m_plugin(plugin)
     , m_win(mw)
-    , m_buildWidget(nullptr)
     , m_proc(this)
-    , m_stdOut()
-    , m_stdErr()
-    , m_buildCancelled(false)
     // NOTE this will not allow spaces in file names.
     // e.g. from gcc: "main.cpp:14: error: cannot convert ‘std::string’ to ‘int’ in return"
     // e.g. from gcc: "main.cpp:14:8: error: cannot convert ‘std::string’ to ‘int’ in return"
     // e.g. from icpc: "main.cpp(14): error: no suitable conversion function from "std::string" to "int" exists"
     // e.g. from clang: ""main.cpp(14,8): fatal error: 'boost/scoped_array.hpp' file not found"
-    , m_filenameDetector(QStringLiteral("(?<filename>(?:[a-np-zA-Z]:[\\\\/])?[^\\s:(]+)[:(](?<line>\\d+)[,:]?(?<column>\\d+)?[):]*\\s*(?<message>.*)"))
+    , m_filenameDetector(QStringLiteral("(?<filename>(?:[a-np-zA-Z]:[\\\\/])?[^\\s:[(]+)[:(](?<line>\\d+)[,:]?(?<column>\\d+)?[):]*\\s*(?<message>.*)"))
     , m_newDirDetector(QStringLiteral("make\\[.+\\]: .+ '(.*)'"))
     , m_diagnosticsProvider(mw, this)
 {
@@ -351,7 +342,14 @@ KateBuildView::KateBuildView(KateBuildPlugin *plugin, KTextEditor::MainWindow *m
 
     m_buildWidget = new QWidget(m_toolView);
     m_buildUi.setupUi(m_buildWidget);
+    m_buildUi.u_outpTopLayout->setSpacing(m_buildWidget->style()->pixelMetric(QStyle::PM_LayoutHorizontalSpacing));
+    m_buildUi.u_outpTopLayout->setContentsMargins(m_buildWidget->style()->pixelMetric(QStyle::PM_LayoutLeftMargin),
+                                                  m_buildWidget->style()->pixelMetric(QStyle::PM_LayoutTopMargin),
+                                                  m_buildWidget->style()->pixelMetric(QStyle::PM_LayoutRightMargin),
+                                                  m_buildWidget->style()->pixelMetric(QStyle::PM_LayoutBottomMargin));
+
     m_targetsUi = new TargetsUi(this, m_buildUi.u_tabWidget);
+    m_buildUi.u_tabWidget->setDocumentMode(true);
     m_buildUi.u_tabWidget->insertTab(0, m_targetsUi, i18nc("Tab label", "Target Settings"));
     m_buildUi.u_tabWidget->setCurrentWidget(m_targetsUi);
     m_buildUi.u_tabWidget->setTabsClosable(true);
@@ -423,9 +421,11 @@ KateBuildView::KateBuildView(KateBuildPlugin *plugin, KTextEditor::MainWindow *m
         auto bg = QColor::fromRgba(theme.editorColor(KSyntaxHighlighting::Theme::EditorColorRole::BackgroundColor));
         auto fg = QColor::fromRgba(theme.textColor(KSyntaxHighlighting::Theme::TextStyle::Normal));
         auto sel = QColor::fromRgba(theme.editorColor(KSyntaxHighlighting::Theme::EditorColorRole::TextSelection));
+        auto linkBg = fg;
         auto errBg = QColor::fromRgba(theme.editorColor(KSyntaxHighlighting::Theme::EditorColorRole::MarkError));
         auto warnBg = QColor::fromRgba(theme.editorColor(KSyntaxHighlighting::Theme::EditorColorRole::MarkWarning));
         auto noteBg = QColor::fromRgba(theme.editorColor(KSyntaxHighlighting::Theme::EditorColorRole::MarkBookmark));
+        linkBg.setAlpha(15);
         errBg.setAlpha(30);
         warnBg.setAlpha(30);
         noteBg.setAlpha(30);
@@ -436,17 +436,19 @@ KateBuildView::KateBuildView(KateBuildPlugin *plugin, KTextEditor::MainWindow *m
         pal.setColor(QPalette::HighlightedText, fg);
         m_buildUi.textBrowser->setPalette(pal);
         m_buildUi.textBrowser->document()->setDefaultStyleSheet(QStringLiteral("a{text-decoration:none;}"
-                                                                               "a:link{color:%1;}\n"
-                                                                               ".err-text {color:%1; background-color: %2;}"
-                                                                               ".warn-text {color:%1; background-color: %3;}"
-                                                                               ".note-text {color:%1; background-color: %4;}"
+                                                                               "a:link{color:%1; background-color: %2;}\n"
+                                                                               ".err-text {color:%1; background-color: %3;}"
+                                                                               ".warn-text {color:%1; background-color: %4;}"
+                                                                               ".note-text {color:%1; background-color: %5;}"
                                                                                "pre{margin:0px;}")
                                                                     .arg(fg.name(QColor::HexArgb))
+                                                                    .arg(linkBg.name(QColor::HexArgb))
                                                                     .arg(errBg.name(QColor::HexArgb))
                                                                     .arg(warnBg.name(QColor::HexArgb))
                                                                     .arg(noteBg.name(QColor::HexArgb)));
         slotUpdateTextBrowser();
     };
+    updateEditorColors(KTextEditor::Editor::instance());
     connect(KTextEditor::Editor::instance(), &KTextEditor::Editor::configChanged, this, updateEditorColors);
 
     connect(m_buildUi.buildAgainButton, &QPushButton::clicked, this, &KateBuildView::slotBuildPreviousTarget);
@@ -481,8 +483,9 @@ KateBuildView::KateBuildView(KateBuildPlugin *plugin, KTextEditor::MainWindow *m
     // Whenever a project target is changed, queue a saving of the targets
     m_saveProjTargetsTimer.setSingleShot(true);
     connect(&m_saveProjTargetsTimer, &QTimer::timeout, this, &KateBuildView::saveProjectTargets);
-    connect(&m_targetsUi->targetsModel, &TargetModel::projectTargetChanged, this, [this]() {
+    connect(&m_targetsUi->targetsModel, &TargetModel::projectTargetChanged, this, [this](const QString &projectBaseDir) {
         if (!m_addingProjTargets) {
+            m_saveProjectTargetDirs.insert(projectBaseDir);
             m_saveProjTargetsTimer.start(1);
         }
     });
@@ -560,7 +563,7 @@ void KateBuildView::readSessionConfig(const KConfigGroup &cg)
     }
 
     // Add project targets, if any
-    addProjectTargets();
+    updateProjectTargets();
 
     // pre-select the last active target or the first target of the first set
     int prevTargetSetRow = cg.readEntry(QStringLiteral("Active Target Index"), 0);
@@ -635,7 +638,7 @@ static Diagnostic createDiagnostic(int line, int column, const QString &message,
 {
     Diagnostic d;
     d.message = message;
-    d.source = DiagnosticsPrefix;
+    d.source = QStringLiteral("katebuild");
     d.severity = severity;
     d.range = KTextEditor::Range(KTextEditor::Cursor(line - 1, column - 1), 0);
     return d;
@@ -674,11 +677,11 @@ void KateBuildView::addError(const KateBuildView::OutputLine &err)
 }
 
 /******************************************************************/
-void KateBuildView::updateDiagnostics(Diagnostic diagnostic, const QUrl uri)
+void KateBuildView::updateDiagnostics(Diagnostic diagnostic, QUrl uri)
 {
     FileDiagnostics fd;
-    fd.uri = uri;
-    fd.diagnostics.append(diagnostic);
+    fd.uri = std::move(uri);
+    fd.diagnostics.append(std::move(diagnostic));
     Q_EMIT m_diagnosticsProvider.diagnosticsAdded(fd);
 }
 
@@ -730,7 +733,7 @@ void KateBuildView::clearBuildResults()
     m_buildUi.textBrowser->clear();
     m_stdOut.clear();
     m_stdErr.clear();
-    m_pendingHtmlOutput = u"<pre>"_s;
+    m_pendingHtmlOutput.clear();
     m_scrollStopLine = -1;
     m_numOutputLines = 0;
     m_numNonUpdatedLines = 0;
@@ -774,13 +777,33 @@ bool KateBuildView::startProcess(const QString &dir, const QString &command)
     QFont font = Utils::editorFont();
     m_buildUi.textBrowser->setFont(font);
 
+    QModelIndex ind = m_targetsUi->targetsView->currentIndex();
+    QString targetSet = ind.data(TargetModel::TargetSetNameRole).toString();
+
     // set working directory
     m_makeDir = dir;
     m_makeDirStack.push(m_makeDir);
 
     if (!QFile::exists(m_makeDir)) {
-        sendError(i18n("Cannot run command: %1\nWork path does not exist: %2", command, m_makeDir));
-        return false;
+        // The build directory does not exist ask if it should be created
+        QMessageBox msgBox(m_win->window());
+        msgBox.setWindowTitle(i18n("Create directory"));
+        msgBox.setTextFormat(Qt::RichText);
+        msgBox.setText(
+            i18n("The configured working directory <b>%1</b> for the targetset <b>%2</b> does not exist.<br><br>"
+                 "Create the directory?",
+                 m_makeDir,
+                 targetSet));
+        msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
+        msgBox.setDefaultButton(QMessageBox::Yes);
+
+        if (msgBox.exec() == QMessageBox::Yes && !QDir().mkpath(m_makeDir)) {
+            displayMessage(i18n("Failed to create the directory <b>%1</b>", m_makeDir), KTextEditor::Message::Error);
+        }
+
+        if (!QFile::exists(m_makeDir)) {
+            return false;
+        }
     }
 
     // chdir used by QProcess will resolve symbolic links.
@@ -886,7 +909,7 @@ KateBuildView::CompileCommands KateBuildView::parseCompileCommandsFile(const QSt
             qCDebug(KTEBUILD) << "parseCompileCommandsFile(): got empty entry at " << i << " !";
             continue; // should not happen
         }
-        res.commands[filename] = {dir, command};
+        res.commands[filename] = {.workingDir = dir, .command = command};
     }
 
     return res;
@@ -1061,6 +1084,7 @@ bool KateBuildView::buildCurrentTarget()
     // When adding new placeholders, also update the tooltip in TargetHtmlDelegate::createEditor()
     if (buildCmd.contains(QLatin1String("%f")) || buildCmd.contains(QLatin1String("%d")) || buildCmd.contains(QLatin1String("%n"))) {
         if (docFInfo.absoluteFilePath().isEmpty()) {
+            sendError(i18n("Cannot make substitution. No open file or the current file is untitled!"));
             return false;
         }
 
@@ -1177,7 +1201,7 @@ QModelIndex KateBuildView::createCMakeTargetSet(QModelIndex setIndex, const QStr
         }
     }
 
-    setIndex = m_targetsUi->targetsModel.insertTargetSetAfter(setIndex, name, cmakeFA.getBuildDir(), true, cmakeConfig);
+    setIndex = m_targetsUi->targetsModel.insertTargetSetAfter(setIndex, name, cmakeFA.getBuildDir(), true, cmakeConfig, cmakeFA.getSourceDir());
     const QModelIndex targetSetIndex = setIndex;
 
     setIndex = m_targetsUi->targetsModel.addCommandAfter(setIndex,
@@ -1289,7 +1313,7 @@ void KateBuildView::displayBuildResult(const QString &msg, KTextEditor::Message:
     }
 
     delete m_infoMessage;
-    m_infoMessage = new KTextEditor::Message(xi18nc("@info", "<title>Make Results:</title><nl/>%1", msg), level);
+    m_infoMessage = new KTextEditor::Message(xi18nc("@info", "<title>Build Finished:</title>%1", msg), level);
     m_infoMessage->setWordWrap(false);
     m_infoMessage->setPosition(KTextEditor::Message::BottomInView);
     m_infoMessage->setAutoHide(5000);
@@ -1316,6 +1340,31 @@ void KateBuildView::displayMessage(const QString &msg, KTextEditor::Message::Mes
     kv->document()->postMessage(m_infoMessage);
 }
 
+/******************************************************************/
+void KateBuildView::displayProgress(const QString &msg, KTextEditor::Message::MessageType level)
+{
+    KTextEditor::View *kv = m_win->activeView();
+    if (!kv) {
+        return;
+    }
+
+    // delete the message if needed
+    if (m_progressMessage && (m_progressMessage->view() != kv || m_progressMessage->messageType() != level)) {
+        delete m_progressMessage;
+    }
+
+    if (!m_progressMessage) {
+        m_progressMessage = new KTextEditor::Message(msg, level);
+        m_progressMessage->setWordWrap(false);
+        m_progressMessage->setPosition(KTextEditor::Message::BottomInView);
+        m_progressMessage->setAutoHide(100000000);
+        m_progressMessage->setAutoHideMode(KTextEditor::Message::Immediate);
+        m_progressMessage->setView(kv);
+        kv->document()->postMessage(m_progressMessage);
+    } else {
+        m_progressMessage->setText(msg);
+    }
+}
 /******************************************************************/
 void KateBuildView::slotProcExited(int exitCode, QProcess::ExitStatus)
 {
@@ -1323,6 +1372,7 @@ void KateBuildView::slotProcExited(int exitCode, QProcess::ExitStatus)
     m_buildUi.u_tabWidget->setTabIcon(1, QIcon::fromTheme(QStringLiteral("format-justify-left")));
     m_buildUi.cancelBuildButton->setEnabled(false);
     m_buildUi.buildAgainButton->setEnabled(true);
+    delete m_progressMessage;
 
     QString buildStatus =
         i18n("Build <b>%1</b> completed. %2 error(s), %3 warning(s), %4 note(s)", m_currentlyBuildingTarget, m_numErrors, m_numWarnings, m_numNotes);
@@ -1354,6 +1404,8 @@ void KateBuildView::slotProcExited(int exitCode, QProcess::ExitStatus)
     }
     m_buildUi.buildStatusLabel->setText(buildStatus);
 
+    m_pendingHtmlOutput += buildStatus;
+
     if (buildSuccess && m_runAfterBuild) {
         m_runAfterBuild = false;
         slotRunAfterBuild();
@@ -1428,7 +1480,7 @@ void KateBuildView::slotRunAfterBuild()
 
 QString KateBuildView::toOutputHtml(const KateBuildView::OutputLine &out)
 {
-    QString htmlStr;
+    QString htmlStr = u"<pre>"_s;
     if (!out.file.isEmpty()) {
         htmlStr += u"<a href=\"%1:%2:%3\">"_s.arg(out.file).arg(out.lineNr).arg(out.column);
     }
@@ -1447,19 +1499,20 @@ QString KateBuildView::toOutputHtml(const KateBuildView::OutputLine &out)
         break;
     }
     htmlStr += out.lineStr.toHtmlEscaped();
-    htmlStr += u"\n</span>"_s;
+    htmlStr += u"</span>"_s;
     if (!out.file.isEmpty()) {
         htmlStr += u"</a>"_s;
     }
+    htmlStr += u"</pre>\n"_s;
+
     return htmlStr;
 }
 
 void KateBuildView::slotUpdateTextBrowser()
 {
-    // move the text, effectively clearing the pending buffer
-    QString html = std::move(m_pendingHtmlOutput);
-    html += u"</pre>"_s;
-    m_pendingHtmlOutput = u"<pre>"_s;
+    if (m_pendingHtmlOutput.isEmpty()) {
+        return;
+    }
 
     QTextBrowser *edit = m_buildUi.textBrowser;
     // Get the scroll position to restore it if not at the end
@@ -1481,7 +1534,7 @@ void KateBuildView::slotUpdateTextBrowser()
 
         if (m_scrollStopLine != -1) {
             if (pxPerLine > 1) {
-                int stopLine = std::max(m_scrollStopLine - 4, 0);
+                int stopLine = std::max(m_scrollStopLine - 6, 0);
                 scrollValuePx = stopLine * pxPerLine;
             } else {
                 // Fallback add one empty line
@@ -1500,12 +1553,24 @@ void KateBuildView::slotUpdateTextBrowser()
     // Add the new lines
     QTextCursor cursor = saveCursor;
     cursor.movePosition(QTextCursor::End);
-    cursor.insertHtml(html);
+    // NOTE: The insertHTML() is tricky as we do not have control over the internal structures.
+    // We add this <pre/> tag to add a new line for the next iteration so that we do not get two separate lines
+    // written on the same line.
+    m_pendingHtmlOutput += u"<pre/>"_s;
+    cursor.insertHtml(m_pendingHtmlOutput);
+    m_pendingHtmlOutput.clear();
     // Restore selection and scroll position
     edit->setTextCursor(saveCursor);
 
     m_numNonUpdatedLines = 0;
     edit->verticalScrollBar()->setValue(scrollValuePx);
+
+    if (!m_progress.isEmpty()) {
+        KTextEditor::Message::MessageType type = m_numErrors != 0 ? KTextEditor::Message::Error
+            : m_numWarnings != 0                                  ? KTextEditor::Message::Warning
+                                                                  : KTextEditor::Message::Information;
+        displayProgress(m_progress, type);
+    }
 }
 
 /******************************************************************/
@@ -1517,6 +1582,9 @@ void KateBuildView::slotReadReadyStdOut()
     l.remove(QLatin1Char('\r'));
     m_stdOut += l;
 
+    bool outputVisible = m_buildUi.u_tabWidget->currentIndex() == 1 && m_toolView->isVisible();
+    m_progress.clear();
+    static const QRegularExpression progressReg(u"(?<progress>\\[\\d+/\\d+\\]|\\[\\s*\\d+%\\]).*"_s);
     // handle one line at a time
     int end = -1;
     int start = 0;
@@ -1546,6 +1614,14 @@ void KateBuildView::slotReadReadyStdOut()
                 m_scrollStopLine = m_numOutputLines;
             }
         }
+        if (!outputVisible && m_plugin->m_showBuildProgress) {
+            QRegularExpressionMatch match = progressReg.matchView(line);
+            if (match.hasMatch()) {
+                m_progress = match.captured(u"progress"_s);
+            }
+        } else {
+            delete m_progressMessage;
+        }
         start = end + 1;
     }
 
@@ -1626,8 +1702,13 @@ void KateBuildView::slotPluginViewCreated(const QString &name, QObject *pluginVi
     // add view
     if (pluginView && name == QLatin1String("kateprojectplugin")) {
         m_projectPluginView = pluginView;
-        addProjectTargets();
-        connect(pluginView, SIGNAL(projectMapChanged()), this, SLOT(slotProjectMapChanged()), Qt::UniqueConnection);
+        updateProjectTargets();
+        pluginView->disconnect(this);
+        connect(pluginView, SIGNAL(projectMapEdited()), this, SLOT(slotProjectMapEdited()), Qt::QueuedConnection);
+        connect(pluginView, SIGNAL(pluginProjectAdded(QString, QString)), this, SLOT(slotProjectMapEdited()), Qt::QueuedConnection);
+        connect(pluginView, SIGNAL(pluginProjectRemoved(QString, QString)), this, SLOT(slotProjectMapEdited()), Qt::QueuedConnection);
+        connect(pluginView, SIGNAL(projectMapChanged()), this, SLOT(slotProjectChanged()), Qt::QueuedConnection);
+        slotProjectChanged();
     }
 }
 
@@ -1637,23 +1718,34 @@ void KateBuildView::slotPluginViewDeleted(const QString &name, QObject *)
     // remove view
     if (name == QLatin1String("kateprojectplugin")) {
         m_projectPluginView = nullptr;
-        m_targetsUi->targetsModel.deleteProjectTargerts();
+        m_targetsUi->targetsModel.deleteProjectTargetsExcept();
+        slotProjectChanged();
     }
 }
 
 /******************************************************************/
-void KateBuildView::slotProjectMapChanged()
+void KateBuildView::slotProjectMapEdited()
 {
     // only do stuff with valid project
     if (!m_projectPluginView) {
+        m_targetsUi->targetsModel.deleteProjectTargetsExcept();
         return;
     }
-    m_targetsUi->targetsModel.deleteProjectTargerts();
-    addProjectTargets();
+    updateProjectTargets();
 }
 
 /******************************************************************/
-void KateBuildView::addProjectTargets()
+void KateBuildView::slotProjectChanged()
+{
+    if (m_projectPluginView) {
+        m_targetsUi->currentProjectBaseDir = m_projectPluginView->property("projectBaseDir").toString();
+    } else {
+        m_targetsUi->currentProjectBaseDir.clear();
+    }
+}
+
+/******************************************************************/
+void KateBuildView::updateProjectTargets()
 {
     // only do stuff with a valid project
     if (!m_projectPluginView) {
@@ -1662,92 +1754,95 @@ void KateBuildView::addProjectTargets()
 
     BoolTrueLocker locker(m_addingProjTargets);
 
-    // Delete any old project plugin targets
-    m_targetsUi->targetsModel.deleteProjectTargerts();
+    const auto allProjects = m_projectPluginView->property("allProjects").toMap();
 
     const QModelIndex projRootIndex = m_targetsUi->targetsModel.projectRootIndex();
-    const QString projectsBaseDir = m_projectPluginView->property("projectBaseDir").toString();
+    // Delete any old project plugin targets
+    m_targetsUi->targetsModel.deleteProjectTargetsExcept(allProjects.keys());
 
-    // Read the targets from the override if available
-    const QString userOverride = projectsBaseDir + u"/.kateproject.build"_s;
-    if (QFile::exists(userOverride)) {
-        // We have user modified commands
-        QFile file(userOverride);
-        if (file.open(QIODevice::ReadOnly)) {
-            const QByteArray jsonString = file.readAll();
-            QJsonParseError error;
-            const QJsonDocument doc = QJsonDocument::fromJson(jsonString, &error);
-            if (error.error != QJsonParseError::NoError) {
-                qWarning() << "Could not parse the provided Json";
-            } else {
-                QJsonObject root = doc.object();
-                QJsonArray targetSets = root[u"target_sets"_s].toArray();
-                for (int i = targetSets.size() - 1; i >= 0; --i) {
-                    QJsonObject set = targetSets[i].toObject();
-                    if (set[u"loaded_via_cmake"_s].toBool()) {
-                        QString buildDir = set[u"directory"_s].toString();
-                        QTimer::singleShot(0, this, [this, buildDir]() {
-                            loadCMakeTargets(buildDir);
-                        });
-                        targetSets.removeAt(i);
+    for (const auto &projectBaseDir : allProjects.keys()) {
+        const QString projectName = allProjects.value(projectBaseDir).toString();
+        QJsonObject root;
+        // Read the targets from the override if available
+        const QString userOverride = projectBaseDir + u"/.kateproject.build"_s;
+        if (QFile::exists(userOverride)) {
+            // We have user modified commands
+            QFile file(userOverride);
+            if (file.open(QIODevice::ReadOnly)) {
+                const QByteArray jsonString = file.readAll();
+                QJsonParseError error;
+                const QJsonDocument doc = QJsonDocument::fromJson(jsonString, &error);
+                if (error.error != QJsonParseError::NoError) {
+                    qWarning() << "Could not parse the provided Json";
+                } else {
+                    root = doc.object();
+                    QJsonArray targetSets = root[u"target_sets"_s].toArray();
+                    for (int i = targetSets.size() - 1; i >= 0; --i) {
+                        QJsonObject set = targetSets[i].toObject();
+                        if (set[u"loaded_via_cmake"_s].toBool()) {
+                            QString buildDir = set[u"directory"_s].toString();
+                            QTimer::singleShot(0, this, [this, buildDir]() {
+                                loadCMakeTargets(buildDir);
+                            });
+                            targetSets.removeAt(i);
+                        }
                     }
+                    root.insert(u"target_sets"_s, targetSets);
+                    root.remove(u"Auto_generated");
                 }
-                root[u"target_sets"_s] = targetSets;
-                QModelIndex addedIndex = m_targetsUi->targetsModel.insertAfter(projRootIndex, root);
-                if (addedIndex != projRootIndex) {
-                    return;
-                    // The user file overrides the rest
-                    // TODO maybe add heuristics for when to re-read the project data
-                }
+            }
+        } else {
+            // do we have a valid map for build settings?
+            QVariantMap projectMap;
+            QMetaObject::invokeMethod(m_projectPluginView,
+                                      "projectMapFor",
+                                      Qt::DirectConnection,
+                                      Q_RETURN_ARG(QVariantMap, projectMap),
+                                      Q_ARG(QString, projectBaseDir));
+
+            // ignore it if we have no targets there
+            const QVariantMap buildMap = projectMap.value(u"build"_s).toMap();
+            const QVariantList targets = buildMap.value(u"targets"_s).toList();
+            if (targets.isEmpty()) {
+                continue;
+            }
+
+            // handle build directory as relative to project file, if possible, see bug 413306
+            QString projectsBuildDir = buildMap.value(u"directory"_s).toString();
+            projectsBuildDir = QDir(projectBaseDir).absoluteFilePath(projectsBuildDir);
+            QJsonObject jsonTargetSet;
+            jsonTargetSet.insert(u"cmake_config"_s, QString());
+            jsonTargetSet.insert(u"directory"_s, projectsBuildDir);
+            jsonTargetSet.insert(u"loaded_via_cmake"_s, false);
+            jsonTargetSet.insert(u"name"_s, projectMap.value(u"name"_s).toString());
+
+            const QString defaultTarget = buildMap.value(u"default_target"_s).toString();
+            QJsonArray jsonTargets;
+            for (const QVariant &targetVariant : targets) {
+                QJsonObject jsonTarget;
+
+                const QVariantMap targetMap = targetVariant.toMap();
+                jsonTarget.insert(u"name"_s, targetMap[u"name"_s].toString());
+                jsonTarget.insert(u"build_cmd"_s, targetMap[u"build_cmd"_s].toString());
+                jsonTarget.insert(u"run_cmd"_s, targetMap[u"run_cmd"_s].toString());
+                jsonTargets.append(jsonTarget);
+            }
+            jsonTargetSet.insert(u"targets", jsonTargets);
+            QJsonArray jsonTargetSets;
+            jsonTargetSets.append(jsonTargetSet);
+            root.insert(u"target_sets"_s, jsonTargetSets);
+        }
+
+        QJsonObject currentRoot = m_targetsUi->targetsModel.projectTargetsToJsonObj(projectBaseDir);
+        if (currentRoot != root) {
+            // TODO Create an update projectarget function that only updates the targets and does not delete and recreate
+            m_targetsUi->targetsModel.deleteProjectTargets(projectBaseDir);
+            QModelIndex addedIndex = m_targetsUi->targetsModel.insertAfter(projRootIndex, root, projectBaseDir);
+            if (addedIndex != projRootIndex) {
+                continue;
             }
         }
     }
-
-    // query new project map
-    const QVariantMap projectMap = m_projectPluginView->property("projectMap").toMap();
-
-    // do we have a valid map for build settings?
-    // ignore it if we have no targets there
-    const QVariantMap buildMap = projectMap.value(QStringLiteral("build")).toMap();
-    const QVariantList targetsets = buildMap.value(QStringLiteral("targets")).toList();
-    if (targetsets.isEmpty()) {
-        return;
-    }
-
-    // handle build directory as relative to project file, if possible, see bug 413306
-    QString projectsBuildDir = buildMap.value(QStringLiteral("directory")).toString();
-    QString projectName = m_projectPluginView->property("projectName").toString();
-    if (!projectsBaseDir.isEmpty()) {
-        projectsBuildDir = QDir(projectsBaseDir).absoluteFilePath(projectsBuildDir);
-    }
-
-    const QModelIndex set = m_targetsUi->targetsModel.insertTargetSetAfter(projRootIndex, projectName, projectsBuildDir);
-    const QString defaultTarget = buildMap.value(QStringLiteral("default_target")).toString();
-
-    for (const QVariant &targetVariant : targetsets) {
-        const QVariantMap targetMap = targetVariant.toMap();
-        const QString tgtName = targetMap[QStringLiteral("name")].toString();
-        const QString buildCmd = targetMap[QStringLiteral("build_cmd")].toString();
-        const QString runCmd = targetMap[QStringLiteral("run_cmd")].toString();
-
-        if (tgtName.isEmpty() || buildCmd.isEmpty()) {
-            continue;
-        }
-        QPersistentModelIndex idx = m_targetsUi->targetsModel.addCommandAfter(set, tgtName, buildCmd, runCmd);
-        if (tgtName == defaultTarget) {
-            // A bit of backwards compatibility, move the "default" target to the top
-            while (idx.row() > 0) {
-                m_targetsUi->targetsModel.moveRowUp(idx);
-            }
-        }
-    }
-
-    const auto index = m_targetsUi->proxyModel.mapFromSource(set);
-    if (index.isValid()) {
-        m_targetsUi->targetsView->expand(index);
-    }
-
-    // FIXME read CMakePresets.json for more build target sets
 }
 
 /******************************************************************/
@@ -1757,40 +1852,50 @@ void KateBuildView::saveProjectTargets()
     if (!m_projectPluginView) {
         return;
     }
-    const QModelIndex projRootIndex = m_targetsUi->targetsModel.projectRootIndex();
-    const QString projectsBaseDir = m_projectPluginView->property("projectBaseDir").toString();
-    const QString userOverride = projectsBaseDir + u"/.kateproject.build"_s;
 
-    QJsonObject root = m_targetsUi->targetsModel.indexToJsonObj(projRootIndex);
-    root[u"Auto_generated"_s] = u"This file is auto-generated. Any extra tags or formatting will be lost"_s;
-    // Do not save CMake auto generated targets
-    QJsonArray targetSets = root[u"target_sets"_s].toArray();
-    for (int i = targetSets.size() - 1; i >= 0; --i) {
-        QJsonObject set = targetSets[i].toObject();
-        if (set[u"loaded_via_cmake"_s].toBool()) {
-            // These will be loaded again when loadCMakeTargets is called
-            set[u"targets"_s] = {};
-            targetSets[i] = set;
+    const QSet<QString> dirs = m_saveProjectTargetDirs;
+    m_saveProjectTargetDirs = {};
+
+    // traverse over all project directories
+    for (const auto &projectBaseDir : dirs) {
+        const QString userOverride = projectBaseDir + u"/.kateproject.build"_s;
+        // get the target sets for this project
+        QJsonObject root = m_targetsUi->targetsModel.projectTargetsToJsonObj(projectBaseDir);
+        if (root.isEmpty()) {
+            // if there are not target remove the override
+            QFile::remove(userOverride);
+            continue;
+        }
+        root[u"Auto_generated"] = u"This file is auto-generated. Any extra tags or formatting will be lost"_s;
+        // Do not save CMake auto generated targets
+        QJsonArray targetSets = root[u"target_sets"_s].toArray();
+        for (int i = targetSets.size() - 1; i >= 0; --i) {
+            QJsonObject set = targetSets[i].toObject();
+            if (set[u"loaded_via_cmake"_s].toBool()) {
+                // These will be loaded again when loadCMakeTargets is called
+                set[u"targets"_s] = {};
+                targetSets[i] = set;
+            }
         }
-    }
 
-    if (targetSets.isEmpty()) {
-        // Nothing left for the userOverride -> remove the file
-        QFile::remove(userOverride);
-        // There are no target-sets left, try reloading the original project-targets
-        addProjectTargets();
-        return;
-    }
-    root[u"target_sets"_s] = targetSets;
+        if (targetSets.isEmpty()) {
+            // Nothing left for the userOverride -> remove the file
+            QFile::remove(userOverride);
+            // There are no target-sets left, try reloading the original project-targets
+            updateProjectTargets();
+            return;
+        }
+        root[u"target_sets"_s] = targetSets;
 
-    QFile file(userOverride);
-    if (!file.open(QIODevice::ReadWrite | QIODevice::Truncate)) {
-        displayMessage(i18n("Cannot save build targets in: %1", userOverride), KTextEditor::Message::Error);
-        return;
+        QFile file(userOverride);
+        if (!file.open(QIODevice::ReadWrite | QIODevice::Truncate)) {
+            displayMessage(i18n("Cannot save build targets in: %1", userOverride), KTextEditor::Message::Error);
+            return;
+        }
+        QJsonDocument doc(root);
+        file.write(doc.toJson());
+        file.close();
     }
-    QJsonDocument doc(root);
-    file.write(doc.toJson());
-    file.close();
 }
 
 /******************************************************************/
@@ -1798,7 +1903,7 @@ bool KateBuildView::eventFilter(QObject *obj, QEvent *event)
 {
     switch (event->type()) {
     case QEvent::KeyPress: {
-        QKeyEvent *ke = static_cast<QKeyEvent *>(event);
+        auto *ke = static_cast<QKeyEvent *>(event);
         if ((obj == m_toolView) && (ke->key() == Qt::Key_Escape)) {
             m_win->hideToolView(m_toolView);
             event->accept();
@@ -1807,7 +1912,7 @@ bool KateBuildView::eventFilter(QObject *obj, QEvent *event)
         break;
     }
     case QEvent::ShortcutOverride: {
-        QKeyEvent *ke = static_cast<QKeyEvent *>(event);
+        auto *ke = static_cast<QKeyEvent *>(event);
         if (ke->matches(QKeySequence::Copy)) {
             m_buildUi.textBrowser->copy();
             event->accept();
@@ -1820,7 +1925,7 @@ bool KateBuildView::eventFilter(QObject *obj, QEvent *event)
         break;
     }
     case QEvent::KeyRelease: {
-        QKeyEvent *ke = static_cast<QKeyEvent *>(event);
+        auto *ke = static_cast<QKeyEvent *>(event);
         if (ke->matches(QKeySequence::Copy) || ke->matches(QKeySequence::SelectAll)) {
             event->accept();
             return true;
@@ -1840,7 +1945,7 @@ void KateBuildView::handleEsc(QEvent *e)
         return;
     }
 
-    QKeyEvent *k = static_cast<QKeyEvent *>(e);
+    auto *k = static_cast<QKeyEvent *>(e);
     if (k->key() == Qt::Key_Escape && k->modifiers() == Qt::NoModifier) {
         if (m_toolView->isVisible()) {
             m_win->hideToolView(m_toolView);
