@@ -24,15 +24,19 @@
 #include "kateviewmanager.h"
 
 #include <KConfigGroup>
+#include <KLocalization>
 #include <KLocalizedString>
 #include <KMessageBox>
-#include <KPluralHandlingSpinBox>
 #include <KSharedConfig>
 
 #include <QCheckBox>
+#include <QCloseEvent>
 #include <QComboBox>
 #include <QDesktopServices>
 #include <QDialogButtonBox>
+#include <QDir>
+#include <QFileInfo>
+#include <QFontDatabase>
 #include <QFrame>
 #include <QGroupBox>
 #include <QLabel>
@@ -42,6 +46,7 @@
 #include <QScreen>
 #include <QScrollArea>
 #include <QScrollBar>
+#include <QSpinBox>
 #include <QTimer>
 #include <QVBoxLayout>
 
@@ -102,17 +107,40 @@ void KateConfigDialog::addBehaviorPage()
     KSharedConfig::Ptr config = KSharedConfig::openConfig();
     KConfigGroup cgGeneral = KConfigGroup(config, QStringLiteral("General"));
 
-    QFrame *generalFrame = new QFrame(this);
+    auto *generalFrame = new QFrame(this);
     KPageWidgetItem *item = addScrollablePage(generalFrame, i18n("Behavior"));
     m_allPages.insert(item);
     item->setHeader(i18n("Behavior Options"));
     item->setIcon(QIcon::fromTheme(QStringLiteral("preferences-system-windows-behavior")));
 
-    QVBoxLayout *layout = new QVBoxLayout(generalFrame);
+    auto *layout = new QVBoxLayout(generalFrame);
+
+    // PATH setting
+    if (KateApp::isKate()) {
+        auto *buttonGroup = new QGroupBox(i18n("PATH"), generalFrame);
+        auto *vbox = new QVBoxLayout(buttonGroup);
+        layout->addWidget(buttonGroup);
+
+        auto label = new QLabel(buttonGroup);
+        label->setText(
+            i18n("List of %1 seperated directories where Kate will look for programs to run. This list will be prepended to your PATH environment variable. "
+                 "E.g '/home/USER/myapps/bin/:/home/USER/.local/bin'",
+                 QDir::listSeparator()));
+        label->setWordWrap(true);
+
+        m_pathEdit = new QLineEdit(buttonGroup);
+        m_pathEdit->setObjectName(QStringLiteral("katePATHedit"));
+        m_pathEdit->setFont(QFontDatabase::systemFont(QFontDatabase::FixedFont));
+        m_pathEdit->setText(cgGeneral.readEntry("Kate PATH", QString()));
+        connect(m_pathEdit, &QLineEdit::textChanged, this, &KateConfigDialog::slotChanged);
+
+        vbox->addWidget(label);
+        vbox->addWidget(m_pathEdit);
+    }
 
     // GROUP with the one below: "Behavior"
-    QGroupBox *buttonGroup = new QGroupBox(i18n("&Behavior"), generalFrame);
-    QVBoxLayout *vbox = new QVBoxLayout;
+    auto *buttonGroup = new QGroupBox(i18n("&Behavior"), generalFrame);
+    auto *vbox = new QVBoxLayout;
     layout->addWidget(buttonGroup);
 
     // shall we try to behave like some SDI application
@@ -171,9 +199,9 @@ void KateConfigDialog::addBehaviorPage()
     buttonGroup->setLayout(vbox);
 
     if (KateApp::isKate()) {
-        QGroupBox *buttonGroup = new QGroupBox(i18n("&Sidebars"), generalFrame);
+        auto *buttonGroup = new QGroupBox(i18n("&Sidebars"), generalFrame);
         layout->addWidget(buttonGroup);
-        QVBoxLayout *vbox = new QVBoxLayout;
+        auto *vbox = new QVBoxLayout;
         m_syncSectionSizeWithSidebarTabs = new QCheckBox(i18n("Sync section size with tab positions"), buttonGroup);
         m_syncSectionSizeWithSidebarTabs->setChecked(cgGeneral.readEntry("Sync section size with tab positions", false));
         m_syncSectionSizeWithSidebarTabs->setToolTip(
@@ -274,6 +302,14 @@ void KateConfigDialog::addBehaviorPage()
     connect(m_openNewTabInFrontOfCurrent, &QCheckBox::toggled, this, &KateConfigDialog::slotChanged);
     vbox->addWidget(m_openNewTabInFrontOfCurrent);
 
+    m_cycleThroughTabs = new QCheckBox(
+        i18nc("'Next Tab'/'Previous Tab' is an existing action name. Please use the same text here as it is used in that action's translation",
+              "Allow cycling through tabs. If unchecked, the action 'Next Tab'/'Previous Tab' will not go to first/last tab after reaching the end"),
+        this);
+    m_cycleThroughTabs->setChecked(cgGeneral.readEntry("Cycle To First Tab", true));
+    connect(m_cycleThroughTabs, &QCheckBox::toggled, this, &KateConfigDialog::slotChanged);
+    vbox->addWidget(m_cycleThroughTabs);
+
     layout->addWidget(buttonGroup);
 
     buttonGroup = new QGroupBox(i18n("&Mouse"), generalFrame);
@@ -347,6 +383,19 @@ void KateConfigDialog::addBehaviorPage()
     hlayout->addWidget(m_diagnosticsLimit);
     connect(m_diagnosticsLimit, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged), this, &KateConfigDialog::slotChanged);
 
+    buttonGroup = new QGroupBox(i18n("Context"), generalFrame);
+    vbox = new QVBoxLayout(buttonGroup);
+    hlayout = new QHBoxLayout;
+    vbox->addLayout(hlayout);
+    layout->addWidget(buttonGroup);
+    m_hintViewEnabled = new QCheckBox(i18n("Enable tool view"), buttonGroup);
+    m_hintViewEnabled->setToolTip(
+        i18n("A tool view to display context associated with the current "
+             "cursor position (LSP hints, diagnostics)"));
+    hlayout->addWidget(m_hintViewEnabled);
+    m_hintViewEnabled->setChecked(cgGeneral.readEntry("Enable Context ToolView", false));
+    connect(m_hintViewEnabled, &QCheckBox::toggled, this, &KateConfigDialog::slotChanged);
+
     layout->addStretch(1); // :-] works correct without autoadd
 }
 
@@ -355,8 +404,8 @@ void KateConfigDialog::addSessionPage()
     KSharedConfig::Ptr config = KSharedConfig::openConfig();
     KConfigGroup cgGeneral = KConfigGroup(config, QStringLiteral("General"));
 
-    QWidget *sessionsPage = new QWidget(this);
-    auto item = addScrollablePage(sessionsPage, i18n("Session"));
+    auto *sessionsPage = new QWidget(this);
+    KPageWidgetItem *item = addScrollablePage(sessionsPage, i18n("Session"));
     m_allPages.insert(item);
     item->setHeader(i18n("Session Management"));
     item->setIcon(QIcon::fromTheme(QStringLiteral("view-history")));
@@ -370,19 +419,17 @@ void KateConfigDialog::addSessionPage()
     // meta infos days
     sessionConfigUi.daysMetaInfos->setMaximum(180);
     sessionConfigUi.daysMetaInfos->setSpecialValueText(i18nc("The special case of 'Delete unused meta-information after'", "(never)"));
-    sessionConfigUi.daysMetaInfos->setSuffix(ki18ncp("The suffix of 'Delete unused meta-information after'", " day", " days"));
+    KLocalization::setupSpinBoxFormatString(sessionConfigUi.daysMetaInfos,
+                                            ki18ncp("The suffix of 'Delete unused meta-information after'", "%v day", "%v days"));
     sessionConfigUi.daysMetaInfos->setValue(KateApp::self()->documentManager()->getDaysMetaInfos());
-    connect(sessionConfigUi.daysMetaInfos,
-            static_cast<void (KPluralHandlingSpinBox::*)(int)>(&KPluralHandlingSpinBox::valueChanged),
-            this,
-            &KateConfigDialog::slotChanged);
+    connect(sessionConfigUi.daysMetaInfos, &QSpinBox::valueChanged, this, &KateConfigDialog::slotChanged);
 
     // restore view  config
     sessionConfigUi.restoreVC->setChecked(cgGeneral.readEntry("Restore Window Configuration", true));
     connect(sessionConfigUi.restoreVC, &QCheckBox::toggled, this, &KateConfigDialog::slotChanged);
 
     sessionConfigUi.spinBoxRecentFilesCount->setValue(recentFilesMaxCount());
-    connect(sessionConfigUi.spinBoxRecentFilesCount, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged), this, &KateConfigDialog::slotChanged);
+    connect(sessionConfigUi.spinBoxRecentFilesCount, &QSpinBox::valueChanged, this, &KateConfigDialog::slotChanged);
 
     QString sesStart(cgGeneral.readEntry("Startup Session", "manual"));
     if (sesStart == QLatin1String("new")) {
@@ -429,8 +476,8 @@ void KateConfigDialog::addSessionPage()
 
 void KateConfigDialog::addPluginsPage()
 {
-    QFrame *page = new QFrame(this);
-    QVBoxLayout *vlayout = new QVBoxLayout(page);
+    auto *page = new QFrame(this);
+    auto *vlayout = new QVBoxLayout(page);
     vlayout->setContentsMargins(0, 0, 0, 0);
     vlayout->setSpacing(0);
 
@@ -438,7 +485,7 @@ void KateConfigDialog::addPluginsPage()
     vlayout->addWidget(m_configPluginPage);
     connect(m_configPluginPage, &KateConfigPluginPage::changed, this, &KateConfigDialog::slotChanged);
 
-    auto item = addScrollablePage(page, i18n("Plugins"));
+    KPageWidgetItem *item = addScrollablePage(page, i18n("Plugins"));
     m_allPages.insert(item);
     item->setHeader(i18n("Plugin Manager"));
     item->setIcon(QIcon::fromTheme(QStringLiteral("preferences-plugin")));
@@ -454,11 +501,11 @@ void KateConfigDialog::addFeedbackPage()
     vlayout->setSpacing(0);
 
     m_userFeedbackWidget = new KUserFeedback::FeedbackConfigWidget(page);
-    m_userFeedbackWidget->setFeedbackProvider(&KateApp::self()->userFeedbackProvider());
+    m_userFeedbackWidget->setFeedbackProvider(KateApp::self()->userFeedbackProvider());
     connect(m_userFeedbackWidget, &KUserFeedback::FeedbackConfigWidget::configurationChanged, this, &KateConfigDialog::slotChanged);
     vlayout->addWidget(m_userFeedbackWidget);
 
-    auto item = addScrollablePage(page, i18n("User Feedback"));
+    KPageWidgetItem *item = addScrollablePage(page, i18n("User Feedback"));
     m_allPages.insert(item);
     item->setHeader(i18n("User Feedback"));
     item->setIcon(QIcon::fromTheme(QStringLiteral("preferences-desktop-locale")));
@@ -610,6 +657,7 @@ void KateConfigDialog::slotApply()
         cg.writeEntry("Allow Tab Scrolling", m_tabsScrollable->isChecked());
         cg.writeEntry("Elide Tab Text", m_tabsElided->isChecked());
         cg.writeEntry("Open New Tab To The Right Of Current", m_openNewTabInFrontOfCurrent->isChecked());
+        cg.writeEntry("Cycle To First Tab", m_cycleThroughTabs->isChecked());
 
         cg.writeEntry("Diff Show Style", m_diffStyle->currentIndex());
 
@@ -618,6 +666,8 @@ void KateConfigDialog::slotApply()
         cg.writeEntry("Diagnostics Limit", m_diagnosticsLimit->value());
         cg.writeEntry("Output With Date", m_withDate->isChecked());
 
+        cg.writeEntry("Enable Context ToolView", m_hintViewEnabled->isChecked());
+
         // patch document modified warn state
         const QList<KTextEditor::Document *> &docs = KateApp::self()->documentManager()->documentList();
         for (KTextEditor::Document *doc : docs) {
@@ -633,9 +683,46 @@ void KateConfigDialog::slotApply()
 
 #ifdef WITH_KUSERFEEDBACK
         // set current active mode + write back the config for future starts
-        KateApp::self()->userFeedbackProvider().setTelemetryMode(m_userFeedbackWidget->telemetryMode());
-        KateApp::self()->userFeedbackProvider().setSurveyInterval(m_userFeedbackWidget->surveyInterval());
+        KateApp::self()->userFeedbackProvider()->setTelemetryMode(m_userFeedbackWidget->telemetryMode());
+        KateApp::self()->userFeedbackProvider()->setSurveyInterval(m_userFeedbackWidget->surveyInterval());
 #endif
+
+        if (KateApp::isKate()) {
+            const QString existingPATH = cg.readEntry("Kate PATH", QString());
+            const QString newPATH = m_pathEdit->text();
+            const QChar seperator = QDir::listSeparator();
+            // check if it changed
+            if (existingPATH != newPATH) {
+                // validate entries, check they exist
+                const QStringList paths = newPATH.split(seperator, Qt::SkipEmptyParts);
+                QStringList validatedPaths;
+                for (const auto &p : paths) {
+                    if (!QFileInfo::exists(p)) {
+                        QMessageBox::warning(this, i18n("Bad PATH value"), i18n("The path '%1' doesn't exist", p));
+                    } else {
+                        validatedPaths.append(p);
+                    }
+                }
+
+                // Remove existing user set path
+                QString oldPATH = QString::fromUtf8(qgetenv("PATH"));
+                if (oldPATH.startsWith(existingPATH)) {
+                    oldPATH.remove(0, existingPATH.length());
+                    if (oldPATH.startsWith(seperator)) {
+                        oldPATH.remove(0, 1);
+                    }
+                }
+
+                // prepend to PATH
+                const QString p = validatedPaths.join(seperator);
+                const QString newPATH = p + seperator + oldPATH;
+                // putenv the new PATH
+                qputenv("PATH", newPATH.toUtf8());
+
+                // write out the setting
+                cg.writeEntry("Kate PATH", p);
+            }
+        }
     }
 
     for (const PluginPageListItem &plugin : std::as_const(m_pluginPages)) {
@@ -719,12 +806,12 @@ KPageWidgetItem *KateConfigDialog::addScrollablePage(QWidget *page, const QStrin
 {
     // inspired by KPageWidgetItem *KConfigDialogPrivate::addPageInternal(QWidget *page, const QString &itemName, const QString &pixmapName, const QString
     // &header)
-    QWidget *frame = new QWidget(this);
-    QVBoxLayout *boxLayout = new QVBoxLayout(frame);
+    auto *frame = new QWidget(this);
+    auto *boxLayout = new QVBoxLayout(frame);
     boxLayout->setContentsMargins(0, 0, 0, 0);
     boxLayout->setContentsMargins(0, 0, 0, 0);
 
-    QScrollArea *scroll = new QScrollArea(frame);
+    auto *scroll = new QScrollArea(frame);
     scroll->setFrameShape(QFrame::NoFrame);
     scroll->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
     scroll->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
